#!/usr/bin/env python
# -*- mode: python; coding: koi8-r -*-

import gtk
from gtk import TRUE, FALSE

import string, gzip

from pybookreader.utils import string_encode, default_description_length
from pybookreader.bookmarks import Book


##----------------------------------------------------------------------
##----------------------------------------------------------------------

class plain_text:

    def __init__(self):
        pass

##----------------------------------------------------------------------

    def new_book(self, filename):
        """Create new book object"""

        book = Book()
        book.filename = filename
        book.encoding = 'koi8-r'
        book.file_type = 'plain_text'
        book.description = self.book_description(book)

        return book

##----------------------------------------------------------------------

    def load_file(self, buffer, book):
        """Load file in buffer"""

        data = self._read_file(book.filename)

        # delete invalid (\0) chars
        tbl = string.maketrans('', '')
        data = string.translate(data, tbl, '\0')

        data = string_encode(data, book.encoding, 'utf-8')

        buffer.set_text(data)

    def _read_file(self, filename, length=None, offset=None):

        fd = open(filename)
        if offset:
            fd.seek(offset)
        if length:
            data = fd.read(length)
        else:
            data = fd.read()
        fd.close()

        return data

    def preview(self, buffer, book, offset):
        """Create preview: load part of file in buffer"""

        data = self._read_file(book.filename, length=1000, offset=offset)
        data = string_encode(data, book.encoding, 'utf-8')
        buffer.set_text(data)


##----------------------------------------------------------------------

    def book_description(self, book):
        """Create description of book"""

        data = self._read_file(book.filename, default_description_length)
        data = data.decode(book.encoding, 'replace')

        tbl = string.maketrans('', '')
        desc = string.translate(data.encode('utf-8', 'replace'), tbl, '\0')

        desc += ' ...'

        return desc


##----------------------------------------------------------------------

    def mark_description(self, book, offset): # ??? (not used)
        """Create description of mark"""
        print 'mark_description'

        data = self._read_file(book.filename)
        data = data[offset:offset+default_description_length]
        data = data.decode(book.encoding, 'replace')

        tbl = string.maketrans('', '')
        desc = string.translate(data.encode('utf-8', 'replace'), tbl, '\0')

        desc += ' ...'

        return desc

##----------------------------------------------------------------------
##----------------------------------------------------------------------

class gzipped_plain_text(plain_text):

    def __init__(self):
        pass

    def _read_file(self, filename, length=None, offset=None):

        fd = gzip.open(filename)
        if offset:
            fd.seek(offset)
        if length:
            data = fd.read(length)
        else:
            data = fd.read()
        fd.close()

        return data

##----------------------------------------------------------------------

    def new_book(self, filename):
        """Create new book object"""

        book = Book()
        book.filename = filename
        book.encoding = 'koi8-r'
        book.file_type = 'gzipped_plain_text'
        book.description = self.book_description(book)

        return book

##----------------------------------------------------------------------
##----------------------------------------------------------------------
